@@ -1,4 +1,4 @@
-/* $Header: /src/pub/tcsh/sh.proc.h,v 3.11 2004/11/23 02:10:49 christos Exp $ */
+/* $Header: /p/tcsh/cvsroot/tcsh/sh.proc.h,v 3.12 2006/01/12 19:55:38 christos Exp $ */
 /*
  * sh.proc.h: Process data structures and variables
  */
@@ -116,7 +116,6 @@ struct process {
 #define	AREASON		0200
 
 EXTERN struct process proclist IZERO_STRUCT;/* list head of all processes */
-EXTERN int    pnoprocesses IZERO;	/* pchild found nothing to wait for */
 
 EXTERN struct process *pholdjob IZERO;	/* one level stack of current jobs */
 
@@ -126,8 +125,4 @@ EXTERN struct process *pprevious IZERO;	/* previous job in table */
 
 EXTERN int   pmaxindex IZERO;		/* current maximum job index */
 
-#ifndef BSDTIMES
-EXTERN int    timesdone;	/* shtimes buffer full ? */
-#endif /* BSDTIMES */
-
 #endif /* _h_sh_proc */
