@@ -1,4 +1,4 @@
-/*$Header: /src/pub/tcsh/win32/ps.c,v 1.4 2004/05/19 18:22:28 christos Exp $*/
+/*$Header: /p/tcsh/cvsroot/tcsh/win32/ps.c,v 1.9 2006/03/14 01:22:58 mitr Exp $*/
 /*-
  * Copyright (c) 1980, 1991 The Regents of the University of California.
  * All rights reserved.
@@ -90,14 +90,14 @@ BOOL CALLBACK enum_wincb(HWND hwnd,LPARAM nump) {
 
 	if (!GetWindowThreadProcessId(hwnd,&pid))
 		return TRUE;
-	
+
 	for (i =0;i < nump;i++) {
 		if (processlist[i].pid == pid){
 			processlist[i].hwnd = hwnd;
 			if (processlist[i].title[0] !=0)
 				break;;
 			GetWindowText(hwnd,processlist[i].title,
-						sizeof(processlist[i].title));
+					sizeof(processlist[i].title));
 			break;
 		}
 	}
@@ -109,12 +109,12 @@ BOOL CALLBACK enum_wincb2(HWND hwnd,LPARAM pidtokill) {
 
 	if (!GetWindowThreadProcessId(hwnd,&pid))
 		return TRUE;
-	if (pid == pidtokill){
+	if (pid == (DWORD)pidtokill){
 		ghwndtokillbywm_close = hwnd;
 		PostMessage( hwnd, WM_CLOSE, 0, 0 );
 		return TRUE;
 	}
-	
+
 	return TRUE;
 }
 int kill_by_wm_close(int pid)  {
@@ -134,9 +134,9 @@ DWORD Win95Lister(void) {
 	hsnap = createSnapshot(TH32CS_SNAPPROCESS,0);
 	if (hsnap == INVALID_HANDLE_VALUE)
 		return 0;
-	
-//	if (processlist)
-//		p_free(processlist);
+
+	//	if (processlist)
+	//		p_free(processlist);
 
 	pe.dwSize = sizeof(PROCESSENTRY32);
 	if (proc32First(hsnap,&pe) ) {
@@ -144,14 +144,14 @@ DWORD Win95Lister(void) {
 		if (!processlist)
 			goto done;
 
-		while(1) {
-			lstrcpy(processlist[nump].exename,pe.szExeFile);
+		do {
+			StringCbCopy(processlist[nump].exename,
+					sizeof(processlist[nump].exename),pe.szExeFile);
+
 			processlist[nump].title[0] = 0;
 			processlist[nump].pid = pe.th32ProcessID;
 			nump++;
-			if (!proc32Next(hsnap,&pe))
-				break;
-		}
+		}while(proc32Next(hsnap,&pe));
 	}
 done:
 	CloseHandle(hsnap);
@@ -163,15 +163,15 @@ done:
 }
 
 DWORD NTLister(void) {
-	
+
 	DWORD procs[200],dummy,ignore;
 	HANDLE hproc;
 	HMODULE hmod;
 	unsigned int i;
-	
 
-//	if (processlist)
-//		p_free(processlist);
+
+	//	if (processlist)
+	//		p_free(processlist);
 
 	if (!enum_processes(procs,sizeof(procs),&dummy) ) {
 		return 0;
@@ -188,18 +188,20 @@ DWORD NTLister(void) {
 		processlist[i].pid = procs[i];
 		processlist[i].title[0] = 0;
 		hproc = OpenProcess(PROCESS_QUERY_INFORMATION |PROCESS_VM_READ,
-							FALSE,procs[i]);
+				FALSE,procs[i]);
 		if (hproc) {
 			if (enum_process_modules(hproc,&hmod,sizeof(hmod),&ignore)) {
 				GetModuleBaseNameA(hproc,hmod, processlist[i].exename,MAX_PATH);
 			}
 			else
-				lstrcpy(processlist[i].exename,"(unknown)");
+				StringCbCopy(processlist[i].exename,
+						sizeof(processlist[i].exename),"(unknown)");
 			CloseHandle(hproc);
 		}
 		else
-			lstrcpy(processlist[i].exename,"(unknown)");
-		
+			StringCbCopy(processlist[i].exename,
+					sizeof(processlist[i].exename),"(unknown)");
+
 	}
 	if (g_dowindows) {
 		EnumWindows(enum_wincb,(LPARAM)dummy);
@@ -207,273 +209,57 @@ DWORD NTLister(void) {
 	return dummy;
 }
 
-DWORD NTLister_old_style( void) {
-
-	DWORD                        rc;
-	HKEY                         hKeyNames;
-	DWORD                        dwType;
-	DWORD                        dwSize;
-	LPBYTE                       buf = NULL;
-	char                         szSubKey[1024];
-	LANGID                       lid;
-	LPSTR                        p;
-	LPSTR                        p2;
-	PPERF_DATA_BLOCK             pPerf;
-	PPERF_OBJECT_TYPE            pObj;
-	PPERF_INSTANCE_DEFINITION    pInst;
-	PPERF_COUNTER_BLOCK          pCounter;
-	PPERF_COUNTER_DEFINITION     pCounterDef;
-	DWORD                        i;
-	DWORD                        dwProcessIdTitle;
-	DWORD                        dwProcessIdCounter;
-	char                         szProcessName[MAX_PATH];
-	unsigned long numprocesses=0;
-
-
-
-	lid = MAKELANGID( LANG_ENGLISH, SUBLANG_NEUTRAL );
-	wsprintf( szSubKey, "%s\\%03x", REGKEY_PERF, lid );
-	rc = RegOpenKeyEx( HKEY_LOCAL_MACHINE, szSubKey,0,KEY_READ,&hKeyNames);
-	if (rc != ERROR_SUCCESS) {
-		goto exit;
-	}
-
-	rc = RegQueryValueEx( hKeyNames, REGSUBKEY_COUNTERS, NULL, &dwType,
-			NULL, &dwSize);
-
-	if (rc != ERROR_SUCCESS) 
-		goto exit;
-
-	buf =  heap_alloc( dwSize );
-	if (buf == NULL) 
-		goto exit;
-
-	rc = RegQueryValueEx( hKeyNames, REGSUBKEY_COUNTERS, NULL, &dwType,
-			buf, &dwSize);
-
-	if (rc != ERROR_SUCCESS) 
-		goto exit;
-
-
-	p = buf;
-	while (*p) {
-		if (p > buf) {
-			for( p2=p-2; isdigit(*p2); p2--) ;
-		}
-		if (lstrcmpi(p, PROCESS_COUNTER) == 0) {
-			for( p2=p-2; isdigit(*p2); p2--) ;
-			lstrcpy( szSubKey, p2+1 );
-		}
-		else if (lstrcmpi(p, PROCESSID_COUNTER) == 0) {
-			for( p2=p-2; isdigit(*p2); p2--) ;
-			dwProcessIdTitle = atol( p2+1 );
-		}
-		p += (lstrlen(p) + 1);
-	}
-
-	heap_free( buf );
-
-
-	dwSize = 65535;
-
-	buf = heap_alloc( dwSize );
-	if (buf == NULL) 
-		goto exit;
-
-
-	while (1) {
-
-		rc = RegQueryValueEx( HKEY_PERFORMANCE_DATA, szSubKey, NULL,
-				&dwType, buf, &dwSize);
-
-		pPerf = (PPERF_DATA_BLOCK) buf;
-
-		if ((rc == ERROR_SUCCESS) &&
-				(dwSize > 0) &&
-				(pPerf)->Signature[0] == (WCHAR)'P' &&
-				(pPerf)->Signature[1] == (WCHAR)'E' &&
-				(pPerf)->Signature[2] == (WCHAR)'R' &&
-				(pPerf)->Signature[3] == (WCHAR)'F' ) {
-			break;
-		}
-
-		if (rc == ERROR_MORE_DATA) {
-			dwSize += 32000;
-			buf = heap_realloc( buf, dwSize );
-		}
-		else {
-			goto exit;
-		}
-	}
-
-	//
-	// set the perf_object_type pointer
-	//
-	pObj = (PPERF_OBJECT_TYPE) ((DWORD)pPerf + pPerf->HeaderLength);
-
-	//
-	// loop thru the performance counter definition records looking
-	// for the process id counter and then save its offset
-	//
-	pCounterDef = (PPERF_COUNTER_DEFINITION) ((DWORD)pObj + pObj->HeaderLength);
-	for (i=0; i<(DWORD)pObj->NumCounters; i++) {
-		if (pCounterDef->CounterNameTitleIndex == dwProcessIdTitle) {
-			dwProcessIdCounter = pCounterDef->CounterOffset;
-			break;
-		}
-		pCounterDef++;
-	}
-
-
-	pInst = (PPERF_INSTANCE_DEFINITION) ((DWORD)pObj + pObj->DefinitionLength);
-
-	numprocesses = pObj->NumInstances;
-
-//	if (processlist)
-//		p_free(processlist);
-
-	processlist = heap_alloc(numprocesses*sizeof(pslist));
-
-	if (!processlist){
-		goto exit;
-	}
-	for (i=0; i<numprocesses; i++) {
-		//
-		// pointer to the process name
-		//
-		p = (LPSTR) ((DWORD)pInst + pInst->NameOffset);
-
-		//
-		// convert it to ascii
-		//
-		rc = WideCharToMultiByte( CP_ACP,
-				0,
-				(LPCWSTR)p,
-				-1,
-				szProcessName,
-				sizeof(szProcessName),
-				NULL,
-				NULL
-								);
-
-		if (!rc) {
-			lstrcpy( processlist[i].exename, "unknown" );
-		}
-		if (!lstrcmpi(szProcessName,"_Total")){
-			numprocesses--;
-			continue;
-		}
-
-		if (lstrlen(szProcessName)+4 <= sizeof(processlist[i].exename)) {
-			lstrcpy( processlist[i].exename, szProcessName );
-			lstrcat( processlist[i].exename, ".exe" );
-		}
-
-		pCounter = (PPERF_COUNTER_BLOCK) ((DWORD)pInst + pInst->ByteLength);
-		processlist[i].pid = *((LPDWORD) ((DWORD)pCounter + dwProcessIdCounter));
-		//        if (processlist[i].pid == 0) {
-		//            processlist[i].pid = (DWORD)-2;
-		//        }
-
-		pInst = (PPERF_INSTANCE_DEFINITION) 
-			((DWORD)pCounter + pCounter->ByteLength);
-	}
-
-	if (g_dowindows) {
-		EnumWindows(enum_wincb,(LPARAM)numprocesses);
-	}
-exit:
-	if (buf) {
-		heap_free( buf );
-	}
-
-	RegCloseKey( hKeyNames );
-	RegCloseKey( HKEY_PERFORMANCE_DATA );
-
-	return numprocesses;
-}
 void init_plister(void) {
 
 
-	if (gdwPlatform == VER_PLATFORM_WIN32_NT && gdwVersion < 5) {
+	hlib = LoadLibrary("kernel32.dll");
+	if (!hlib)
+		return ;
 
-		hlib = LoadLibrary("psapi.dll");
-		
-		if (!hlib){
-			ProcessListFunc =NTLister_old_style;
-			return ;
-		}
 
-		ProcessListFunc = NTLister;
+	ProcessListFunc = Win95Lister;
 
-		enum_processes = (enumproc)GetProcAddress(hlib,"EnumProcesses");
-		enum_process_modules = (enummod)GetProcAddress(hlib,
-							"EnumProcessModules");
-		getfilenameex = (getfilename_ex)GetProcAddress(hlib,
-							"GetModuleFileNameExA");
-		GetModuleBaseNameA = (getbasename)GetProcAddress(hlib,
-						"GetModuleBaseNameA");
-		if (!enum_processes || !enum_process_modules || !getfilenameex){
-			FreeLibrary(hlib);
-			ProcessListFunc = NULL;
-		}
+	proc32First = (walker)GetProcAddress(hlib,"Process32First");
+	proc32Next = (walker)GetProcAddress(hlib,"Process32Next");
+	createSnapshot= (create_snapshot)GetProcAddress(hlib,
+			"CreateToolhelp32Snapshot");
+
+	FreeLibrary(hlib);
+	if (!proc32First || !proc32Next || !createSnapshot) {
+		ProcessListFunc = NULL;
 	}
-	else{
-		hlib = LoadLibrary("kernel32.dll");
-		if (!hlib)
-			return ;
-
-
-		ProcessListFunc = Win95Lister;
-
-		proc32First = (walker)GetProcAddress(hlib,"Process32First");
-		proc32Next = (walker)GetProcAddress(hlib,"Process32Next");
-		createSnapshot= (create_snapshot)GetProcAddress(hlib,
-											"CreateToolhelp32Snapshot");
-
-		FreeLibrary(hlib);
-		if (!proc32First || !proc32Next || !createSnapshot) {
-			ProcessListFunc = NULL;
-		}
-	}
-
 }
 void dops(Char ** vc, struct command *c) {
-	
+
 	DWORD nump;
 	unsigned int i,k;
 	char **v;
 
+	UNREFERENCED_PARAMETER(c);
+
 	if (!ProcessListFunc)
 		return;
-	gflag = 0;
-	tglob(vc);
-	if (gflag) {
-		vc = globall(vc);
-		if (vc == 0)
-			stderror(ERR_NAME | ERR_NOMATCH);
-	}
-	else
-		vc = gargv = saveblk(vc);
-	trim(vc);
+	vc = glob_all_or_error(vc);
 	v = short2blk(vc);
+	blkfree(vc);
 	for (k = 0; v[k] != NULL ; k++){
 		if ( v[k][0] == '-' ) {
 			if( (v[k][1] == 'W') || (v[k][1] == 'w'))
 				g_dowindows = 1;
 		}
 	}
+	blkfree((Char**)v);
 	nump = ProcessListFunc();
 
 	for(i=0; i< nump; i++) {
 		if (gdwPlatform == VER_PLATFORM_WIN32_NT) 
 			xprintf("%6u  %-20s %-30s\n",processlist[i].pid,
-							processlist[i].exename, 
-							g_dowindows?processlist[i].title:"");
+					processlist[i].exename, 
+					g_dowindows?processlist[i].title:"");
 		else
 			xprintf("0x%08x  %-20s %-30s\n",processlist[i].pid,
-							processlist[i].exename,
-							g_dowindows?processlist[i].title:"");
+					processlist[i].exename,
+					g_dowindows?processlist[i].title:"");
 	}
 	g_dowindows =0;
 
@@ -482,9 +268,9 @@ void dops(Char ** vc, struct command *c) {
 
 }
 static char shutdown_usage[]= {"shutdown -[r|l][f] now\n-r reboots, -l logs\
-off the current user\n-f forces termination of running applications.\n\
-The default action is to shutdown without a reboot.\n\"now\" must be \
-specified to actually shutdown or reboot\n"};
+	off the current user\n-f forces termination of running applications.\n\
+		The default action is to shutdown without a reboot.\n\"now\" must be \
+		specified to actually shutdown or reboot\n"};
 
 void doshutdown(Char **vc, struct command *c) {
 
@@ -499,22 +285,17 @@ void doshutdown(Char **vc, struct command *c) {
 	LUID luid;
 	DWORD cbPrevious = sizeof(TOKEN_PRIVILEGES);
 
+	UNREFERENCED_PARAMETER(c);
+
 	if (gdwPlatform != VER_PLATFORM_WIN32_NT) {
 		stderror(ERR_SYSTEM,"shutdown","Sorry,not supported on win95");
 	}
 
 	shutdown_ok = reboot = shutdown = logoff = 0;
-	gflag = 0;
-	tglob(vc);
-	if (gflag) {
-		vc = globall(vc);
-		if (vc == 0)
-			stderror(ERR_NAME | ERR_NOMATCH);
-	}
-	else
-		vc = gargv = saveblk(vc);
-	trim(vc);
+	vc = glob_all_or_error(vc);
 	v = short2blk(vc);
+	blkfree(vc);
+	cleanup_push((Char **)v, blk_cleanup);
 	for (k = 0; v[k] != NULL ; k++){
 		if ( v[k][0] == '-' ) {
 			ptr = v[k];
@@ -526,52 +307,42 @@ void doshutdown(Char **vc, struct command *c) {
 					reboot =1;
 				else if (*ptr == 'l')
 					logoff =1;
-				else {
-					blkfree((Char **)v);
+				else
 					stderror(ERR_SYSTEM,"Usage",shutdown_usage);
-				}
 				ptr++;
 			}
 		}
-		else if (!lstrcmpi(v[k],"now")) {
+		else if (!_stricmp(v[k],"now")) {
 			shutdown_ok = 1;
 		}
 	}
-	if (k == 0) {
-		blkfree((Char**)v);
+	if (k == 0)
 		stderror(ERR_SYSTEM,"Usage",shutdown_usage);
-	}
 	if (!reboot && !logoff){
 		flags |= EWX_SHUTDOWN;
 		shutdown = 1;
 	}
-	if (reboot && logoff ) {
-		blkfree((Char **)v);
+	if (reboot && logoff )
 		stderror(ERR_SYSTEM,"Usage",shutdown_usage);
-	}
 	if (reboot)
 		flags |= EWX_REBOOT;
 	if (logoff)
 		flags |= EWX_LOGOFF;
 
-	if ((reboot || shutdown) && (!shutdown_ok) ) {
-		blkfree((Char **)v);
+	if ((reboot || shutdown) && (!shutdown_ok) )
 		stderror(ERR_SYSTEM,"shutdown","Specify \"now\" to really shutdown");
-	}
 
 
 	if (!OpenProcessToken(GetCurrentProcess(),
-							TOKEN_ADJUST_PRIVILEGES| TOKEN_QUERY,
-							&hToken) ){
+				TOKEN_ADJUST_PRIVILEGES| TOKEN_QUERY,
+				&hToken) ){
 		make_err_str(GetLastError(),errbuf,128);
-		blkfree((Char **)v);
 		stderror(ERR_SYSTEM,"shutdown failed",errbuf);
 	}
-							
+
 
 	if (!LookupPrivilegeValue(NULL,SE_SHUTDOWN_NAME,&luid)) {
 		make_err_str(GetLastError(),errbuf,128);
-		blkfree((Char **)v);
 		stderror(ERR_SYSTEM,"shutdown failed",errbuf);
 	}
 	tp.PrivilegeCount = 1;
@@ -581,7 +352,6 @@ void doshutdown(Char **vc, struct command *c) {
 	if (!AdjustTokenPrivileges(hToken,FALSE,&tp,sizeof(tp),&tpPrevious,
 				&cbPrevious)){
 		make_err_str(GetLastError(),errbuf,128);
-		blkfree((Char **)v);
 		stderror(ERR_SYSTEM,"shutdown failed",errbuf);
 	}
 	tpPrevious.PrivilegeCount = 1;
@@ -591,12 +361,11 @@ void doshutdown(Char **vc, struct command *c) {
 	if (!AdjustTokenPrivileges(hToken,FALSE,&tpPrevious,cbPrevious,NULL,
 				NULL)){
 		make_err_str(GetLastError(),errbuf,128);
-		blkfree((Char **)v);
 		stderror(ERR_SYSTEM,"shutdown failed",errbuf);
 	}
 	if  (  !ExitWindowsEx(flags,0) ) {
 		make_err_str(GetLastError(),errbuf,128);
-		blkfree((Char **)v);
 		stderror(ERR_SYSTEM,"shutdown failed",errbuf);
 	}
+	cleanup_until((Char **)v);
 }
