@@ -1,4 +1,4 @@
-/*$Header: /src/pub/tcsh/win32/signal.c,v 1.5 2003/11/04 02:02:59 amold Exp $*/
+/*$Header: /p/tcsh/cvsroot/tcsh/win32/signal.c,v 1.11 2006/08/25 17:49:57 christos Exp $*/
 /*-
  * Copyright (c) 1980, 1991 The Regents of the University of California.
  * All rights reserved.
@@ -41,6 +41,7 @@
 #include "forkdata.h"
 #include "signal.h"
 
+#pragma warning(disable:4055)
 
 #define SIGBAD(signo)        ( (signo) <=0 || (signo) >=NSIG) 
 #define fast_sigmember(a,b)  ( (*(a) & (1 << (b-1)) ) )
@@ -68,10 +69,10 @@ static ChildListNode *clist_t; // tail of list
 static CRITICAL_SECTION sigcritter;
 static HANDLE hmainthr;
 static HANDLE hsigsusp;
-static int __is_suspended;
+static int __is_suspended = 0;
 static HANDLE __halarm=0;
 
-extern HANDLE __h_con_alarm,__h_con_int;
+extern HANDLE __h_con_alarm,__h_con_int, __h_con_hup;
 
 // must be done before fork;
 void nt_init_signals(void) {
@@ -89,12 +90,12 @@ void nt_init_signals(void) {
 					0,
 					FALSE,
 					DUPLICATE_SAME_ACCESS)){
-		int err = GetLastError();
-		ExitProcess(0);
+		ExitProcess(GetLastError());
 	}
-	hsigsusp = CreateEvent(NULL,TRUE,FALSE,NULL);
+	hsigsusp = CreateEvent(NULL,FALSE,FALSE,NULL);
 	__h_con_alarm=CreateEvent(NULL,FALSE,FALSE,NULL);
 	__h_con_int=CreateEvent(NULL,FALSE,FALSE,NULL);
+	__h_con_hup=CreateEvent(NULL,FALSE,FALSE,NULL);
 	if (!hsigsusp)
 		abort();
 
@@ -107,6 +108,7 @@ void nt_cleanup_signals(void) {
 	CloseHandle(hsigsusp);
 	CloseHandle(__h_con_alarm);
 	CloseHandle(__h_con_int);
+	CloseHandle(__h_con_hup);
 	CloseHandle(__halarm);
 }
 int sigaddset(sigset_t *set, int signo) {
@@ -146,8 +148,12 @@ void deliver_pending(void) {
 
 		if (temp & 0x01){
 			if (gPending[sig]){
-				gPending[sig]=0;
-				generic_handler(sig);
+				//gPending[sig]=0;
+                do {
+                    dprintf("deliver_pending for sig %d\n",sig);
+                    gPending[sig]--;
+                    generic_handler(sig);
+                }while(gPending[sig] != 0);
 			}
 		}
 		temp >>= 1;
@@ -184,23 +190,23 @@ int sigsuspend(const sigset_t *mask) {
 
 
 	EnterCriticalSection(&sigcritter);
-	__is_suspended =1;
+	__is_suspended++;
 	LeaveCriticalSection(&sigcritter);
 
 	sigprocmask(SIG_SETMASK,mask,&omask);
 
-	WaitForSingleObject(hsigsusp,INFINITE);
-	ResetEvent(hsigsusp);
+    dprintf("suspending main thread susp count %d\n",__is_suspended);
+    do {
+        WaitForSingleObject(hsigsusp,INFINITE);
+    }while(__is_suspended > 0);
 
-	EnterCriticalSection(&sigcritter);
-	__is_suspended =0;
-	LeaveCriticalSection(&sigcritter);
 
 	sigprocmask(SIG_SETMASK,&omask,0);
 	errno = EINTR;
 	return -1;
 
 }
+
 int sigaction(int signo, const struct sigaction *act, struct sigaction *oact) {
 
 	if (SIGBAD(signo)) {
@@ -213,7 +219,8 @@ int sigaction(int signo, const struct sigaction *act, struct sigaction *oact) {
 			oact->sa_mask = 0;
 			oact->sa_flags =0;
 	}
-	if (signo == SIGHUP &&  (act && act->sa_handler == SIG_IGN) && __forked)
+	if ((signo == SIGHUP) && (act && (act->sa_handler == SIG_IGN)) 
+						&& __forked)
 		__nt_child_nohupped = 1;
 	if (act)
 		handlers[signo]=act->sa_handler;
@@ -221,82 +228,18 @@ int sigaction(int signo, const struct sigaction *act, struct sigaction *oact) {
 	return 0;
 	
 }
-#ifdef NEW_SIGNAL_HACK
-#ifndef _M_IX86
-#pragma error("Please undef NEW_SIGNAL_HACK for alphas")
-#endif _M_IX86
-static DWORD sigtodeliver;
-static CONTEXT main_ctx;
-static DWORD saved_eip;
-DWORD WINAPI rsp(void) {
-	main_ctx.Eip = saved_eip;
-	suspend_main_thread();
-	if (!SetThreadContext(hmainthr,&main_ctx)) {
-		int foo = GetLastError();
-	}
-	resume_main_thread();
-	return 0;
-
-}
-void restore_main_ctx(void) {
-	HANDLE ht;
-	DWORD tid;
-	ht = CreateThread(NULL,gdwStackSize,
-				(LPTHREAD_START_ROUTINE)rsp,NULL,0,&tid);
-	CloseHandle(ht);
-	while(1) {
-		Sleep(10);
-	}
-}
-void hack_jump(void) {
-
-	if (handlers[sigtodeliver] == SIG_IGN)
-		restore_main_ctx();
-	if (fast_sigmember(&gBlockMask,sigtodeliver) ){
-		inc_pending(sigtodeliver);
-		restore_main_ctx();
-	}
-	generic_handler(sigtodeliver);
-}
-#endif NEW_SIGNAL_HACK
 int ctrl_handler(DWORD event) {
-#ifdef NEW_SIGNAL_HACK
-
-	sigtodeliver = event+1;
-
-	if (event > CTRL_SHUTDOWN_EVENT)
-		return generic_handler(event +1);
-
-	suspend_main_thread();
-	main_ctx.ContextFlags = CONTEXT_FULL;
-	if (!GetThreadContext(hmainthr,&main_ctx)) {
-		resume_main_thread();
-		return generic_handler(event+1);
-	}
-	saved_eip  = main_ctx.Eip;
-	main_ctx.Eip = (DWORD)hack_jump;
-	if (!SetThreadContext(hmainthr,&main_ctx)) {
-		int foo = GetLastError();
-		resume_main_thread();
-		return generic_handler(event+1);
-	}
-	resume_main_thread();
-	
-	return TRUE;
-#else
-	//
-	// Special hack to prevent signal handlers from executing in their
-	// own threads as that causes havoc with longjmps.
-	//
-	// -amol 4/14/97
 
 	if (event == CTRL_C_EVENT || event == CTRL_BREAK_EVENT) {
 		SetEvent(__h_con_int);
 		return TRUE;
 	}
+	if (event == CTRL_CLOSE_EVENT) {
+		SetEvent(__h_con_hup);
+		return TRUE;
+	}
 
 	return generic_handler(event+1);
-#endif NEW_SIGNAL_HACK
 }
 int generic_handler(DWORD signo) {
 
@@ -384,6 +327,8 @@ int generic_handler(DWORD signo) {
 		case SIGCHLD:
 			if (handlers[signo] != SIG_IGN){
 				if (fast_sigmember(&gBlockMask,signo) ) {
+                    dprintf("inc pending for sig %d count %d\n",signo,
+                        gPending[signo]);
 					inc_pending(signo);
 					blocked=1;
 				}
@@ -395,8 +340,13 @@ int generic_handler(DWORD signo) {
 			ExitProcess(604);
 			break;
 	}
-	if (!blocked && __is_suspended)
-		SetEvent(hsigsusp);
+    if (!blocked && __is_suspended) {
+        EnterCriticalSection(&sigcritter);
+        __is_suspended--;
+        LeaveCriticalSection(&sigcritter);
+        dprintf("releasing suspension is_suspsend = %d\n",__is_suspended);
+        SetEvent(hsigsusp);
+    }
 	return TRUE;
 }
 Sigfunc *_nt_signal(int signal, Sigfunc * handler) {
@@ -423,6 +373,7 @@ int waitpid(pid_t pid, int *statloc, int options) {
 	ChildListNode *temp;
 	int retcode;
 
+	UNREFERENCED_PARAMETER(options);
 	errno = EINVAL;
 	if (pid != -1)
 		return -1;
@@ -472,10 +423,15 @@ unsigned int alarm(unsigned int seconds) {
 	static unsigned int prev_val=0;
 	HANDLE ht;
 	DWORD tid;
+	SECURITY_ATTRIBUTES secd;
+
+	secd.nLength=sizeof(secd);
+	secd.lpSecurityDescriptor=NULL;
+	secd.bInheritHandle=TRUE;
 
 
 	if (!__halarm) {
-		__halarm=CreateEvent(NULL,FALSE,FALSE,NULL);
+		__halarm=CreateEvent(&secd,FALSE,FALSE,NULL);
 	}
 	if(__alarm_set )
 		SetEvent(__halarm);
@@ -487,7 +443,9 @@ unsigned int alarm(unsigned int seconds) {
 	__alarm_set = 1;
 
 	ht = CreateThread(NULL,gdwStackSize,
-				(LPTHREAD_START_ROUTINE)alarm_callback, (void*)seconds,0,&tid);
+				(LPTHREAD_START_ROUTINE)alarm_callback, 
+				(void*)UIntToPtr(seconds),
+				0,&tid);
 	if (ht)
 		CloseHandle(ht);
 	
@@ -520,6 +478,8 @@ end:
 }
 void sig_child_callback(DWORD pid,DWORD exitcode) {
 	
+	DWORD ecode = 0;
+
 	EnterCriticalSection(&sigcritter);
 	add_to_child_list(pid,exitcode);
 	suspend_main_thread();
@@ -529,7 +489,7 @@ void sig_child_callback(DWORD pid,DWORD exitcode) {
 	__try {
 		generic_handler(SIGCHLD);
 	}
-	__except(1) {
+	__except(ecode = GetExceptionCode()) {
 		;
 	}
 	resume_main_thread();
@@ -548,6 +508,7 @@ void sigchild_thread(struct thread_args *args) {
 	CloseHandle(args->hproc);
 	sig_child_callback(args->pid,exitcode);
 	ffree(args);
+    dprintf("exiting sigchild thread for pid %d\n",args->pid);
 }
 void start_sigchild_thread(HANDLE hproc, DWORD pid) {
 
@@ -557,6 +518,7 @@ void start_sigchild_thread(HANDLE hproc, DWORD pid) {
 	args->hproc = hproc;
 	args->pid = pid;
 
+    dprintf("creating sigchild thread for pid %d\n",pid);
 	hthr = CreateThread(NULL,
 							gdwStackSize,
 							(LPTHREAD_START_ROUTINE)sigchild_thread,
@@ -690,8 +652,8 @@ Niceness    Base    Priority class/thread priority
 ****************************************************************************/
 int nice(int niceness) {
 
-    DWORD pclass;
-    int priority;
+    DWORD pclass = IDLE_PRIORITY_CLASS;
+    int priority = THREAD_PRIORITY_NORMAL;
 
     if (niceness < -6 || niceness > 7) {
         errno = EPERM;
@@ -755,12 +717,10 @@ int nice(int niceness) {
     }
 
     if (!SetPriorityClass(GetCurrentProcess(),pclass)){
-        int err = GetLastError() ;
         errno = EPERM;
         return -1;
     }
     if (!SetThreadPriority(GetCurrentThread(),priority)){
-        int err = GetLastError() ;
         errno = EPERM;
         return -1;
     }
